#!/bin/sh -efu

SYS_CONFIG="/etc/openssl/openssl.cnf"
SCRIPT="${1:-./openssl-gost.control}"
STRESS_COUNT=50

workdir="$(mktemp -d --tmpdir openssl-gost-control-check.XXXX)"
if [ -z "$workdir" ]; then
    echo 'ERROR: Unable to create temporary directory.' >&2
    exit 1
fi

on_exit() {
    rm -rf "$workdir"
    echo "Test directory $workdir deleted." >&2
}
trap 'on_exit' EXIT

export OPENSSL_CNF="$workdir/openssl.cnf"
cat "$SYS_CONFIG" >"$OPENSSL_CNF"

quiet_ok=

check_exit_code() {
    local ret=
    local cmd=

    if [ $# -eq 0 ]; then
        cmd="default command"
    else
        cmd="'$1'"
    fi

    ret=0
    "$SCRIPT" "$@" >/dev/null || ret=$?
    if [ $ret -eq 0 ]; then
        [ -n "$quiet_ok" ] || \
            echo "  - $cmd exit code is OK." >&2
    else
        echo "  - $cmd exit code is ERROR." >&2
        return 1
    fi
}

check_status() {
    local ref_status="$1"

    local desc=
    case "$ref_status" in
        enabled)
            desc='enabling'
            ;;
        disabled)
            desc='disabling'
            ;;
        *)
            echo "  - unexpected status '$ref_status'." >&2
            return 1
            ;;
    esac

    local ret=0
    local status="$("$SCRIPT" 'status')" || ret=$?
    if [ $ret -eq 0 ]; then
        if [ "$status" = "$ref_status" ]; then
            [ -n "$quiet_ok" ] || \
                echo "  - $desc GOST is OK." >&2
        else
            echo "  - $desc GOST FAILED." >&2
            exit 3
        fi
    else
        echo "  - status exit code is ERROR after '$ref_status'." >&2
        exit 3
    fi
}

check_diff() {
    local pm="$1"

    local ret=
    local count=
    local ctx=
    local change=

    ret=0
    diff -u "${OPENSSL_CNF}~" "$OPENSSL_CNF" | \
        grep -v -e '^---' -e '^+++' -e '@@' >"$workdir/diff" || \
            ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to write the diff." >&2
        return 1
    fi

    ret=0
    count="$(grep '^[-+]' "$workdir/diff" | wc -l)" || ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to count the diff lines." >&2
        return 1
    fi
    if [ "$count" != '1' ]; then
        echo "  - too much changes!" >&2
        return 1
    fi

    ret=0
    grep -B1 "^[-+]" "$workdir/diff" >"$workdir/hunk" || ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to extract the henk." >&2
        return 1
    fi

    ret=0
    count="$(cat "$workdir/hunk" | wc -l)" || ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to count the hunk lines." >&2
        return 1
    fi
    if [ "$count" != '2' ]; then
        echo "  - too much changes!" >&2
        return 1
    fi

    ret=0
    ctx="$(head -1 "$workdir/hunk")" || ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to extract the context." >&2
        return 1
    fi
    if [ "$ctx" != ' [ engine_section ]' ]; then
        echo "  - unexpected hunk context." >&2
        return 1
    fi

    ret=0
    change="$(tail -1 "$workdir/hunk")" || ret=$?
    if [ $ret -ne 0 ]; then
        echo "ERROR: Unable to extract the change." >&2
        return 1
    fi
    if [ "$change" != "${pm}gost = gost_section" ]; then
        echo "  - unexpected change." >&2
        return 1
    fi

    [ -n "$quiet_ok" ] || \
        echo '  - modification check PASSED.' >&2
}

echo "Checking CLI of the control script:" >&2

check_exit_code 'list'

"$SCRIPT" list | tr '[[:space:]]' '\n' | \
    while read cmd; do
        case "$cmd" in
            enabled|disabled)
                ;;
            *)
                echo "  - unexpected command '$cmd' in 'list'" >&2
                exit 2
                ;;
        esac
    done
echo "  - 'list' commands are OK." >&2

check_exit_code 'help'

"$SCRIPT" help | \
    while read cmd desc; do
        case "$cmd" in
            enabled:|disabled:)
                ;;
            *)
                echo "  - unexpected command '$cmd' in 'help'" >&2
                exit 2
                ;;
        esac
    done
echo "  - 'help' commands are OK." >&2

check_exit_code

check_exit_code 'status'

check_exit_code 'enabled'
check_status 'enabled'

check_exit_code 'disabled'
check_status 'disabled'

echo "Check single modification:" >&2
check_diff '-'

echo "Stress modification test:" >&2
(
    quiet_ok=1
    for i in `seq 1 $STRESS_COUNT`; do
        check_exit_code 'enabled'
        check_status 'enabled'
        check_diff '+'

        check_exit_code 'disabled'
        check_status 'disabled'
        check_diff '-'
    done

    echo '  - modification check PASSED.' >&2
)
